/* colcom -- solve the colorful components problem
   Copyright (C) 2012  Falk Hüffner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  */

#ifndef ILP_HH_INCLUDED
#define ILP_HH_INCLUDED

enum ILPVarType {
  // not actuall working currently (see fixmes)
  // VAR_CONT,
  VAR_BIN,
  VAR_INT,
};

enum ILPConstrSense {
  CONSTR_LEQ,
  CONSTR_EQ,
  CONSTR_GEQ,
};

class CallbackILP {
public:
    virtual ~CallbackILP() { }
    virtual double gap() const = 0;
    virtual void addLazyConstraint(int nz, double rhs, ILPConstrSense sense, const int* cutind, const double* cutval) = 0;
    virtual void addCut           (int nz, double rhs, ILPConstrSense sense, const int* cutind, const double* cutval) = 0;
};

class ILPCallback {
public:
    virtual ~ILPCallback() { }
    virtual void call(CallbackILP& ilp, const double *x, bool isIntegral) = 0;
};

class ILP {
public:
    virtual ~ILP() { }
    
    virtual int numVars() const = 0;
    virtual void setVerbose(bool verbose) = 0;

    virtual void addVariables(int n, const double* obj, const double* lb, const double* ub,
			      const ILPVarType* type, char** name) = 0;
    virtual void addConstraints(int n, int nz, const double* rhs, const ILPConstrSense* sense,
				const int* rmatbeg, const int* rmatind, const double* rmatval) = 0;
    virtual void addMIPstart(const double* x) = 0;
    virtual void addCallback(ILPCallback& callback, bool cuts, bool lazyConstraints) = 0;

    virtual double optimize(double* x) = 0;
};

#endif	// ILP_HH_INCLUDED
