/* colcom -- solve the colorful components problem
   Copyright (C) 2012  Falk Hüffner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  */

#include <ilcplex/cplex.h>

#include "ILP.hh"

class ILPCplex : public ILP {
public:
    ILPCplex();
    ~ILPCplex();

    int numVars() const;
    void setVerbose(bool verbose);

    void addVariables(int n, const double* obj, const double* lb, const double* ub,
		      const ILPVarType* type, char** name);
    void addConstraints(int n, int nz, const double* rhs, const ILPConstrSense* sense,
			const int* rmatbeg, const int* rmatind, const double* rmatval);
    void addMIPstart(const double* x);
    void addCallback(ILPCallback& callback, bool cuts, bool lazyConstraints);

    double optimize(double* x);

private:
    void error(int status, const char* msg);
    CPXENVptr env_;
    CPXLPptr lp_;
};

class CallbackILPCplex : public CallbackILP {
public:
    CallbackILPCplex(CPXCENVptr env, void* cbdata, int wherefrom) :
	env_(env), cbdata_(cbdata), wherefrom_(wherefrom) { }
    double gap() const;
    void addLazyConstraint(int nz, double rhs, ILPConstrSense sense, const int* cutind, const double* cutval);
    void addCut(int nz, double rhs, ILPConstrSense sense, const int* cutind, const double* cutval);

private:
    CPXCENVptr env_;
    void* cbdata_;
    int wherefrom_;
};
