/* colcom -- solve the colorful components problem
   Copyright (C) 2012  Falk Hüffner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  */

extern "C" {
#include <gurobi_c.h>
}

#include "ILP.hh"

class ILPGurobi : public ILP {
public:
    ILPGurobi();
    ~ILPGurobi();

    int numVars() const;
    void setVerbose(bool verbose);

    void addVariables(int n, const double* obj, const double* lb, const double* ub,
		      const ILPVarType* type, char** name);
    void addConstraints(int n, int nz, const double* rhs, const ILPConstrSense* sense,
			const int* rmatbeg, const int* rmatind, const double* rmatval);
    void addMIPstart(const double* x);
    void addCallback(ILPCallback& callback, bool cuts, bool lazyConstraints);

    double optimize(double* x);

    bool useCuts() const { return cuts_; }
    bool useLazyConstraints() const { return lazyConstraints_; }
    ILPCallback* getCallback() const { return callback_; }

private:
    void error(const char* msg) const;
    GRBenv* env_;
    GRBenv* orig_env_;
    GRBmodel* lp_;
    bool cuts_, lazyConstraints_;
    ILPCallback* callback_;
};
