#include <set>
#include <vector>

extern "C" {
#include <caml/alloc.h>
#include <caml/callback.h>
#include <caml/memory.h>
}

#include "CGraph.hh"

std::pair<double, std::set<vertex_t>>
colgenHeuristic(const Graph& g, const std::vector<double>& vertexWeights) {
    static value* colgen_heuristic = nullptr;
    if (!colgen_heuristic)
	colgen_heuristic = caml_named_value("colgen_heuristic");
    value v_weights, v_tuple, v_vs;
    size_t n = g.num_vertices();
    v_weights = caml_alloc(n * Double_wosize, Double_array_tag);
    for (size_t i = 0; i < n; ++i)
	Store_double_field(v_weights, i, vertexWeights[i]);
    v_tuple = caml_callback(*colgen_heuristic, v_weights);
    v_vs = Field(v_tuple, 0);
    double vs_val = Double_val(Field(v_tuple, 1));
    std::set<vertex_t> cluster;
    for (size_t i = 0; i < Wosize_val(v_vs); ++i)
	cluster.insert(Int_val(Field(v_vs, i)));

    return std::make_pair(vs_val, cluster);
}

std::pair<double, std::set<vertex_t>>
independentSetHeuristic(const std::vector<double>& vertexWeights) {
    static value* independent_set_heuristic = nullptr;
    if (!independent_set_heuristic)
	independent_set_heuristic = caml_named_value("independent_set_heuristic");
    value v_weights, v_tuple, v_vs;
    size_t n = vertexWeights.size();
    v_weights = caml_alloc(n * Double_wosize, Double_array_tag);
    for (size_t i = 0; i < n; ++i)
	Store_double_field(v_weights, i, vertexWeights[i]);
    v_tuple = caml_callback(*independent_set_heuristic, v_weights);
    v_vs = Field(v_tuple, 0);
    double vs_val = Double_val(Field(v_tuple, 1));
    std::set<vertex_t> cluster;
    for (size_t i = 0; i < Wosize_val(v_vs); ++i)
	cluster.insert(Int_val(Field(v_vs, i)));

    return std::make_pair(vs_val, cluster);
}
