#ifndef LPCPLEX_HH_INCLUDED
#define LPCPLEX_HH_INCLUDED

#include "LP.hh"

#include <ilcplex/cplex.h>

namespace LP {
    class CplexModel : public Model {
    public:
	CplexModel(Sense sense = Sense::MINIMIZE);
	~CplexModel();
	void setVerbose(bool verbose);
	size_t numVariables() const;
	size_t numConstraints() const;
	Expr objective() const;
	Sense objectiveSense() const;
	Var addVar(double lb, double ub, double obj, const std::string& name);
	Var addIntVar(double lb, double ub, double obj, const std::string& name);
	Constraint addConstraint(const ConstrExpr& c);
	Constraint addConditionalConstraint(bool positive, const Var& v, double bigM,
					    const ConstrExpr& c);
	ConstrExpr constrExpr(const Constraint& c) const;
	void solve();
	Status status() const;
	double objectiveValue() const;
	bool varIsIntegral(const Var& v) const;
	double varlb(const Var& v) const;
	double varub(const Var& v) const;
	double varValue(const Var& v) const;
	double dualValue(const Constraint& c) const;
	void varChangeBounds(const Var& v, double lb, double ub) const;
	void varSetIntegral(const Var& v, bool integral) const;
	void setCoefficient(const Constraint& c, const Var& v, double x) const;
	void setObjectiveCoeff(const Var& v, double c);
	void setObjectiveSense(Sense s);
	void setGoodEnough(double x);
	double goodEnough() const { return goodEnough_; }
	void setEmphasis(Emphasis e);
	void setCutCallback(Callback& callback, bool cuts, bool lazyConstraints);

    private:
	friend int goodEnoughCallback(CPXCENVptr env, void *cbdata, int wherefrom, void* cbhandle);

	std::string varName(int v) const;
	Var doAddVar(double lb, double ub, double obj, const std::string& name, const char* type);

	void error(int status, const std::string& msg) const;
	CPXENVptr env_;
	CPXLPptr lp_;
	double goodEnough_;
	bool minimizing_;
    };

    class CplexCallbackModel : public CallbackModel {
    public:
	CplexCallbackModel(CPXCENVptr env, void* cbdata, int wherefrom) :
	    env_(env), cbdata_(cbdata), wherefrom_(wherefrom) { }
	double gap() const;
	double value(const Var& v) const;
	void addLazyConstraint(const ConstrExpr& c);
	void addUserCut       (const ConstrExpr& c);
    private:
	CPXCENVptr env_;
	void* cbdata_;
	int wherefrom_;
    };
}

#endif
