#ifndef LPGUROBI_HH_INCLUDED
#define LPGUROBI_HH_INCLUDED

#include "LP.hh"

extern "C" {
#include <gurobi_c.h>
}

namespace LP {
    class GurobiModel : public Model {
    public:
	GurobiModel(Sense sense = Sense::MINIMIZE);
	~GurobiModel();
	void setVerbose(bool verbose);
	size_t numVariables() const;
	size_t numConstraints() const;
	Expr objective() const;
	Sense objectiveSense() const;
	Var addVar(double lb, double ub, double obj, const std::string& name);
	Var addIntVar(double lb, double ub, double obj, const std::string& name);
	Constraint addConstraint(const ConstrExpr& c);
	Constraint addConditionalConstraint(bool positive, const Var& v, double bigM,
					    const ConstrExpr& c);
	ConstrExpr constrExpr(const Constraint& c) const;
	void solve();
	Status status() const;
	double objectiveValue() const;
	bool varIsIntegral(const Var& v) const;
	double varlb(const Var& v) const;
	double varub(const Var& v) const;
	double varValue(const Var& v) const;
	double dualValue(const Constraint& c) const;
	void varChangeBounds(const Var& v, double lb, double ub) const;
	void varSetIntegral(const Var& v, bool integral) const;
	void setCoefficient(const Constraint& c, const Var& v, double x) const;
	void setObjectiveCoeff(const Var& v, double c);
	void setObjectiveSense(Sense s);
	void setGoodEnough(double x);
	double goodEnough() const { return goodEnough_; }
	void setEmphasis(Emphasis e);
	void setCutCallback(Callback& callback, bool cuts, bool lazyConstraints);

    private:
	std::string varName(int v) const;
	Var doAddVar(double lb, double ub, double obj, const std::string& name, char type);

	void error(const std::string& msg) const;

	int varIntAttr(const Var& v, const char* attr) const;
	char varCharAttr(const Var& v, const char* attr) const;
	double varDoubleAttr(const Var& v, const char* attr) const;
	const char *varStringAttr(const Var& v, const char* attr) const;
	void update() const { GRBupdatemodel(lp_); addedVars_ = addedConstrs_ = 0; }

	static GRBenv* env_;
	GRBenv* env() const { return GRBgetenv(lp_); }
	GRBmodel* lp_;
	double goodEnough_;
	mutable size_t addedVars_, addedConstrs_;
    };
}

#endif
