/* hcd -- solve the Highly Connected Deletion problem
   Copyright (C) 2012  Falk Hüffner

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  */

extern "C" {
#include <caml/alloc.h>
#include <caml/mlvalues.h>
#include <caml/memory.h>
}

#include <vector>

typedef std::pair<int, int> edge_t;

#if 1

extern "C" {

#define NDEBUG 1
#define SAVECUT 1
#define HO
#include "mincut/graph.h"

void computeCuts(graph *g);

// copy&paste from dimacsParse
graph* parse(int n, int m, const std::vector<edge_t>& edges)
{

#define MAXLINE       100	/* max line length in the input file */
#define ARC_FIELDS      3	/* no of fields in arc line  */
#define NODE_FIELDS     2	/* no of fields in node line  */
#define P_FIELDS        3       /* no of fields in problem line */
#define PROBLEM_TYPE "cut"      /* name of problem type*/

////  int n,m;                      /* number of vertices, arcs */
  graph *g=NULL;                /* the return value */
  int node_min=INT_MAX,		/* minimal no of node  */
      node_max=0,		/* maximal no of nodes */
     *arc_first=NULL,		/* internal array for holding
				   - node degree
				   - position of the first outgoing arc */
      *arc_tail=NULL,		/* internal array: tails of the arcs */
  /* temporary variables carrying no of nodes */
      head, tail, i,

  /* temporary variables carrying no of arcs */
       last, arc_num, arc_new_num;

  node   *head_p, *ndp; /* pointers to the node structure */

  arc    *arc_current=NULL, /* pointer to the arc structure */
         *arc_new,
         *arc_tmp;

  weight_t cap;			/* capacity of the current arc */
#ifdef LONGLONGINTWEIGHTS
  long capr;                    /* we read long longs as longs */
#else
  weight_t capr;                /* we read most types as themselves */
#endif 

  int
////      no_lines=0,		/* no of current input line */
////  no_plines=0,			/* no of problem-lines */
  /*no_nslines=0,            no of node-source-lines */
  /*no_nklines=0,            no of node-source-lines */
  no_alines=0,			/* no of arc-lines */
  pos_current=0;		/* 2*no_alines */

////  char    in_line[MAXLINE],	/* for reading input line */
////    pr_type[3];			/* for reading type of the problem */
////  /*nd;                      source (s) or sink (t) */
////
////  int     err_no;		/* no of detected error */

  /* -------------- error numbers & error messages ---------------- */
#define EN1   0
#define EN2   1
#define EN3   2
#define EN4   3
#define EN6   4
#define EN10  5
#define EN7   6
#define EN8   7
#define EN9   8
#define EN11  9
#define EN12 10
#define EN13 11
#define EN14 12
#define EN16 13
#define EN15 14
#define EN17 15
#define EN18 16
#define EN21 17
#define EN19 18
#define EN20 19
#define EN22 20

////  static char *err_message[] = 
////    { 
////      /* 0*/    "more than one problem line.",
////      /* 1*/    "wrong number of parameters in the problem line.",
////      /* 2*/    "it is not a Max Flow problem line.",
////      /* 3*/    "bad value of a parameter in the problem line.",
////      /* 4*/    "can't obtain enough memory to solve this problem.",
////      /* 5*/    "more than one line with the problem name.",
////      /* 6*/    "can't read problem name.",
////      /* 7*/    "problem description must be before arc description.",
////      /* 8*/    "this parser doesn't support multiply sources and sinks.",
////      /* 9*/    "wrong number of parameters in the node line.",
////      /*10*/    "wrong value of parameters in the node line.",
////      /*11*/    " ",
////      /*12*/    "source and sink descriptions must be before arc descriptions.",
////      /*13*/    "too many arcs in the input.",
////      /*14*/    "wrong number of parameters in the arc line.",
////      /*15*/    "wrong value of parameters in the arc line.",
////      /*16*/    "unknown line type in the input.",
////      /*17*/    "reading error.",
////      /*18*/    "not enough arcs in the input.",
////      /*19*/    "source or sink doesn't have incident arcs.",
////      /*20*/    "can't read anything from the input file."
////	      };
  /* --------------------------------------------------------------- */

  /* The main loop:
     -  reads the line of the input,
     -  analyses its type,
     -  checks correctness of parameters,
     -  puts data to the arrays,
     -  does service functions
     */

////  while ( fgets ( in_line, MAXLINE, fp ) != NULL )
////    {
////      no_lines ++;
////
////      switch (in_line[0])
////	{
////	case 'c':		/* skip lines with comments */
////	case '\n':		/* skip empty lines   */
////	case '\0':		/* skip empty lines at the end of file */
////	  break;
////
////	case 'p':		/* problem description      */
////	  if ( no_plines > 0 )
////	    /* more than one problem line */
////	    { err_no = EN1 ; goto error; }
////
////	  no_plines = 1;
////   
////	  if (
////	      /* reading problem line: 
////		 type of problem, no of vertices, no of arcs */
////	      sscanf ( in_line, "%*c %3s %d %d", pr_type, &n, &m )
////	      != P_FIELDS
////	      )
////	    /*wrong number of parameters in the problem line*/
////	    { err_no = EN2; goto error; }
////
////	  if ( strcmp ( pr_type, PROBLEM_TYPE ) )
////	    /*wrong problem type*/
////	    { err_no = EN3; goto error; }
////
////	  if ( n <= 0  || m <= 0 )
////	    /*wrong value of no of arcs or vertices*/
////	    { err_no = EN4; goto error; }

	  g = makeGraph(n+1, m);
	  /* allocating memory for 'vertices', 'arcs'  and internal arrays */
	  memassert(arc_tail = (int*) calloc ( 2*g->m,   sizeof(int) )); 
	  memassert(arc_first= (int*) calloc ( g->n+1, sizeof(int) ));
	  /* arc_first [ 0 .. n+1 ] = 0 - initialized by calloc */

////	  if ( g->vertexalloc == NULL || g->arcs == NULL || 
////	      arc_first == NULL || arc_tail == NULL )
////	    /* memory is not allocated */
////	    { err_no = EN6; goto error; }
		     
	  /* setting pointer to the first arc */
	  arc_current = g->arcs;

	  node_max = 0;
	  node_min = g->n;

////	  break;
////
////	case 'a':		/* arc description */
////      
////	  if ( no_plines < 1)
////	    { err_no = EN7; goto error; }
////		
////	  if ( no_alines >= g->m )
////	    /*too many arcs on input*/
////	    { err_no = EN16; goto error; }
////		
////	  if (
////	      /* reading an arc description */
////	      sscanf ( in_line, "%*c %d %d %" WT_RD_FORMAT, 
////		      &tail, &head, &capr )
////	      != ARC_FIELDS
////	      ) 
////	    /* arc description is not correct */
////	    { err_no = EN15; goto error; }
////
////	  if ( tail < 0  ||  tail > n  ||
////	      head < 0  ||  head > n  
////	      )
////	    /* wrong value of nodes */
////	    { err_no = EN17; goto error; }

    for (int e = 0; e < m; e++) {        ////
          tail = edges[e].first + 1;     ////
	  head = edges[e].second + 1;    ////
	  capr = 1;

	  /* no of arcs incident to node i is stored in arc_first[i+1] */
	  arc_first[tail + 1] ++; 
	  arc_first[head + 1] ++;

	  /* storing information about the arc */
	  arc_tail[pos_current]        = tail;
	  arc_tail[pos_current+1]      = head;
	  arc_current       -> head    = g->vertexalloc + head;
	  arc_current       -> CAPFIELD    = capr;
	  arc_current       -> rev  = arc_current + 1;
	  ( arc_current + 1 ) -> head    = g->vertexalloc + tail;
	  ( arc_current + 1 ) -> CAPFIELD    = capr;
	  ( arc_current + 1 ) -> rev  = arc_current;

	  /* update minimum and maximum node */
	  if ( head < node_min ) node_min = head;
	  if ( tail < node_min ) node_min = tail;
	  if ( head > node_max ) node_max = head;
	  if ( tail > node_max ) node_max = tail;

	  no_alines   ++;
	  arc_current += 2;
	  pos_current += 2;
////	  break;

////	default:
////	  /* unknown type of line */
////	  err_no = EN18; goto error;
////	  break;
////
////	} /* end of switch */
    } /* end of input loop */

////  /* ----- all is read  or  error while reading ----- */ 
////
////  if ( feof (stdin) == 0 )	/* reading error */
////    { err_no=EN21; goto error; } 
////
////  if ( no_lines == 0 )		/* empty input */
////    { err_no = EN22; goto error; } 
////
////  if ( no_alines < m )		/* not enough arcs */
////    { err_no = EN19; goto error; } 

  /********** ordering arcs - linear time algorithm ***********/

  /* first arc from the first node */
  ( g->vertexalloc + node_min ) -> first = g->arcs;

  /* before below loop arc_first[i+1] is the number of arcs outgoing from i;
     after this loop arc_first[i] is the position of the first 
     outgoing from node i arcs after they would be ordered;
     this value is transformed to pointer and written to node.first[i]
     */
 
  for ( i = node_min + 1; i <= node_max + 1; i ++ ) 
    {
      arc_first[i]          += arc_first[i-1];
      ( g->vertexalloc + i ) -> first = g->arcs + arc_first[i];
    }


  for ( i = node_min; i < node_max; i ++ ) /* scanning all the vertices  
					      exept the last*/
    {

      last = ( ( g->vertexalloc + i + 1 ) -> first ) - g->arcs;
      /* arcs outgoing from i must be cited    
	 from position arc_first[i] to the position
	 equal to initial value of arc_first[i+1]-1  */

      for ( arc_num = arc_first[i]; arc_num < last; arc_num ++ )
	{ tail = arc_tail[arc_num];

	  while ( tail != i )
	    /* the arc no  arc_num  is not in place because arc cited here
	       must go out from i;
	       we'll put it to its place and continue this process
	       until an arc in this position would go out from i */

	    { arc_new_num  = arc_first[tail];
	      arc_current  = g->arcs + arc_num;
	      arc_new      = g->arcs + arc_new_num;
	    
	      /* arc_current must be cited in the position arc_new    
		 swapping these arcs:                                 */

	      head_p               = arc_new -> head;
	      arc_new -> head      = arc_current -> head;
	      arc_current -> head  = head_p;

	      cap                 = arc_new -> CAPFIELD;
	      arc_new -> CAPFIELD     = arc_current -> CAPFIELD;
	      arc_current -> CAPFIELD = cap;

	      if ( arc_new != arc_current -> rev )
		{
		  arc_tmp                = arc_new -> rev;
		  arc_new  -> rev     = arc_current -> rev;
		  arc_current -> rev  = arc_tmp;

		  ( arc_current -> rev ) -> rev = arc_current;
		  ( arc_new     -> rev ) -> rev = arc_new;
		}

	      arc_tail[arc_num] = arc_tail[arc_new_num];
	      arc_tail[arc_new_num] = tail;

	      /* we increase arc_first[tail]  */
	      arc_first[tail] ++ ;

	      tail = arc_tail[arc_num];
	    }
	}
      /* all arcs outgoing from  i  are in place */
    }       

  /* -----------------------  arcs are ordered  ------------------------- */

  /*----------- constructing lists ---------------*/


  for ( ndp=g->vertexalloc+node_min; ndp <= g->vertexalloc + node_max; ndp++)
    ndp -> first = (arc*) NULL;

  for (arc_current =g->arcs+(2*g->m-1); arc_current >= g->arcs; arc_current--)
    {
      arc_num = arc_current - g->arcs;
      tail = arc_tail [arc_num];
      ndp = g->vertexalloc + tail;
      arc_current -> next = ndp -> first;
      ndp -> first = arc_current;
    }


  /* ----------- assigning output values ------------*/
  g->currentN = g->n = node_max - node_min + 1;
  g->vertices = g->vertexalloc + node_min;
  g->sentinelVertex = g->vertices + g->n;
  g->minCap = MAXWEIGHT;

  /* free internal memory */
  free ( arc_first ); free ( arc_tail );

#ifdef CONTRACT
  /* set v->last and a->prev pointers */
  ForAllVertices(g, ndp) 
    if (ndp->first != NULL)
      {
	ndp->first->prev = NULL;
	for (arc_tmp=ndp->first; arc_tmp->next!=NULL; arc_tmp=arc_tmp->next)
	  arc_tmp->next->prev = arc_tmp;
	ndp->last = arc_tmp;
      }
    else
      ndp->last = NULL;

  for(i=0; i<g->n; i++)
    {
      g->nodes[i] = g->vertices+i;
      g->vertices[i].leader = g->vertices+i;
      g->vertices[i].index = i;
    }
  g->nodes[g->n] = NULL;
#ifdef ENABLE_GRAPH_BACKUPS
  for(i=0; i<g->currentM; i++)
    {
      g->currentArcs[i] = g->arcs+i;
      g->arcs[i].index = i;
    }
  g->currentArcs[g->currentM] = NULL;
#endif /* ENABLE_GRAPH_BACKUPS */
#endif /* CONTRACT */

  /* Thanks God! all is done */
  return g;
////
////  /* ---------------------------------- */
//// error:				/* error found reading input */
////
////  printf ( "\nline %d of input - %s\n", 
////	  no_lines, err_message[err_no] );
////
////  exit(1);
}
/* --------------------   end of parser  -------------------*/

#define VERTEX_NUMBER(g, i) ((int)((i)-g->vertices))
static size_t min_cut(size_t n, size_t m, const std::vector<edge_t>& edges, std::vector<int>& cut) {
    graph* g = parse(n, m, edges);

    computeCuts(g);

    node* w;
    int count = 0, in = 1;

    ForAllVertices(g, w)
	if (w->in_cut == 0)
	    count++;
    if (count < g->n / 2)
	in = 0;
    ForAllVertices(g, w)
	if (w->in_cut == in)
	    cut.push_back(VERTEX_NUMBER(g, w));

    size_t result = g->minCap;
    freeGraph(g);
    return result;
}
}

#else

#include <algorithm>

#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/one_bit_color_map.hpp>
#include <boost/graph/stoer_wagner_min_cut.hpp>


static size_t min_cut(size_t n, size_t m, const std::vector<edge_t>& edges, std::vector<int>& cut) {
    typedef boost::adjacency_list<boost::vecS, boost::vecS, boost::undirectedS,
				  boost::no_property, boost::property<boost::edge_weight_t, int> > undirected_graph;
    typedef boost::property_map<undirected_graph, boost::edge_weight_t>::type weight_map_type;
    typedef boost::property_traits<weight_map_type>::value_type weight_type;
    weight_type ws[m];
    for (size_t i = 0; i < m; i++)
	ws[i] = 1;

    undirected_graph g(edges.begin(), edges.begin() + m, ws, n, m);
    BOOST_AUTO(parities, boost::make_one_bit_color_map(num_vertices(g), get(boost::vertex_index, g)));
    int w = boost::stoer_wagner_min_cut(g, get(boost::edge_weight, g), boost::parity_map(parities));
    for (size_t i = 0; i < num_vertices(g); ++i)
	if (get(parities, i))
	    cut.push_back(i);
    return w;
}

#endif

extern "C" {
// external min_cut_c : (int * int) array -> (int * int array)
// n must be >= 2
value c_min_cut(value v_edges) {
    CAMLparam1(v_edges);
    CAMLlocal2(v_tuple, v_cut);

    size_t n = 0;
    size_t m = Wosize_val(v_edges);
    std::vector<edge_t> edges;
    for (size_t i = 0; i < m; i++) {
	value v_edge = Field(v_edges, i);
	size_t u = Int_val(Field(v_edge, 0));
	size_t v = Int_val(Field(v_edge, 1));
	n = std::max(n, u + 1);
	n = std::max(n, v + 1);
	edge_t e;
	e.first = u;
	e.second = v;
	edges.push_back(e);
    }

    std::vector<int> cut;
    int w = min_cut(n, m, edges, cut);

    v_cut = caml_alloc(cut.size(), 0);
    for (size_t i = 0; i < cut.size(); ++i)
	Store_field(v_cut, i, Val_int(cut[i]));
    v_tuple = caml_alloc(2, 0);
    Store_field(v_tuple, 0, Val_int(w));
    Store_field(v_tuple, 1, v_cut);

    CAMLreturn(v_tuple);
}
}

#if 0
int main() {
    edge_t edges[] = {{3, 4}, {3, 6}, {3, 5}, {0, 4}, {0, 1}, {0, 6}, {0, 7},
		      {0, 5}, {0, 2}, {4, 1}, {1, 6}, {1, 5}, {6, 7}, {7, 5}, {5, 2}, {3, 4}};
    std::vector<int> cut;
    int w = min_cut(8, 16, edges, cut);
    std::cout << "cut size: " << w << std::endl;
    for (size_t i = 0; i < cut.size(); i++)
	std::cout << cut[i] << ' ';
    std::cout << std::endl;
    return 0;
}
#endif

